#!/bin/sh
# Copyright(c) 2008 Red Hat Middleware, LLC,
# and individual contributors as indicated by the @authors tag.
# See the copyright.txt in the distribution for a
# full listing of individual contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library in the file COPYING.LIB;
# if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
#
# @author Jean-Frederic Clere
# @author Mladen Turk
#
echo ""
echo "Running : `basename $0` $LastChangedDate: 2015-01-22 07:42:18 -0500 (Thu, 22 Jan 2015) $"
echo ""
echo "Started : `date`"
echo "Common  : $1"
echo "Prefix  : $2"
echo "Output  : $3"
echo "OpenSSL : $4"
echo "Static  : $5"
echo "Sources : $6"
echo ""


# parameters
# $1: Location of the common libraries.
# $2: Destination location.
# $3: Location where to put the binaries.
# $4: Use OpenSSL.
# $5: Use static build.
# $6: Location of the sources.

common_loc=$1
prefix_loc=$2
output_loc=$3
has_openssl_loc=$4
has_static_loc=$5
sources_loc=$6
current_loc=`pwd`

#
# Function from the httpd.spec file
# removed for easy testing.
#        --with-apr=${_prefix} --with-apr-util=${_prefix} \
mpmbuild()
{
mpm=$1; shift
mkdir $mpm
#(cd $mpm
(
bash ./configure \
        --enable-layout=Apache \
        --with-included-apr \
        --prefix=${_sysconfdir}/httpd \
        --exec-prefix=${_prefix} \
        --bindir=${_bindir} \
        --sbindir=${_sbindir} \
        --mandir=${_mandir} \
        --libdir=${_libdir} \
        --sysconfdir=${_sysconfdir}/httpd/conf \
        --includedir=${_includedir}/httpd \
        --libexecdir=${_libdir}/httpd/modules \
        --datadir=${contentdir} \
        --with-installbuilddir=${_libdir}/httpd/build \
        --with-mpm=$mpm \
        --enable-suexec --with-suexec \
        --with-suexec-caller=${suexec_caller} \
        --with-suexec-docroot=${contentdir} \
        --with-suexec-logfile=${_localstatedir}/log/httpd/suexec.log \
        --with-suexec-bin=${_sbindir}/suexec \
        --with-suexec-uidmin=500 --with-suexec-gidmin=100 \
        $* || exit 1
make clean || exit 1
make || exit 1
) || return 1
}

#
# Set needed variables. (defaulted to hacked values).
PREFIX=/opt/jboss/httpd

_sysconfdir=${PREFIX}
_prefix=${PREFIX}
_sbindir=${PREFIX}/sbin
_bindir=${PREFIX}/bin
_mandir=${PREFIX}/man
_libdir=${PREFIX}/lib
_includedir=${PREFIX}/include
contentdir=${PREFIX}/htdocs
suexec_caller=${PREFIX}/bin/suexec
_localstatedir=${PREFIX}/logs

if $has_openssl_loc ; then
  add_conf="--with-ssl=${common_loc} --enable-ssl"
else
  add_conf=""
fi
if $build_iconv; then
  # Use the static iconv we built before.
  LDFLAGS="$LDFLAGS -L${build_common_dir}-static/lib -liconv"
  export LDFLAGS
  # Add the LD_LIBRARY_PATH if we want to use dynamic library. 
  #LD_LIBRARY_PATH="$LD_LIBRARY_PATH:${build_common_dir}-static/lib"
  #export LD_LIBRARY_PATH
fi
if $build_expat; then
  add_conf_sys="--with-expat=${build_common_dir}-static/"
else
  add_conf_sys="--with-expat=builtin"
fi
if $build_zlib; then
  add_conf_sys="$add_conf_sys --with-z=${build_common_dir}-static/"
fi
if $build_pcre; then
  add_conf_sys="$add_conf_sys --with-pcre=${build_common_dir}-static/"
else
  add_conf_sys="$add_conf_sys --with-pcre=builtin"
fi

# Process mod_jk if needed.
native_sources=srclib/`ls srclib | grep httpd-`
cd $native_sources

if $has_jk; then
  add_conf="$add_conf --enable-jk"
  (cd modules/jk; ./configure)
fi

if $has_cluster; then
  case $build_version in
      1.0.*|1.1.*|1.2.*)
        add_conf="$add_conf --enable-proxy-cluster --enable-advertise --enable--slotmem --enable-manager --disable-proxy-balancer"
        slotmem=slotmem
        ;;
      1.3.*)
        add_conf="$add_conf --enable-proxy-cluster --enable-advertise --enable-cluster-slotmem --enable-manager --disable-proxy-balancer"
        slotmem=cluster_slotmem
        ;;
  esac
fi
if $has_bmx; then
  add_conf="$add_conf --enable-bmx --enable-bmx-vhost --enable-bmx-status"
fi

# Ajust some more platform dependent stuff.
case ${BUILD_SYS} in
    linux*)
    add_conf_sys="$add_conf_sys --enable-pie"
    ;;
    hpux*)
    add_conf_sys="$add_conf_sys --without-pgsql"
    ;;
esac
case ${BUILD_CPU} in
    ppc64)
    # XXX: It is also in build.sh but not exported.
    add_conf_sys="$add_conf_sys CFLAGS=-m64"
    ;;
esac
echo "Building prefork and mpm with add_conf: $add_conf $add_conf_sys"

# Build everything and the kitchen sink with the prefork build
# removed because of ldap dependencies.
#        --enable-ldap --enable-authnz-ldap \
mpmbuild prefork \
        --enable-mods-shared=all \
        ${add_conf} \
        ${add_conf_sys} \
        --enable-proxy \
        --enable-cache --enable-mem-cache \
        --enable-file-cache --enable-disk-cache \
        --enable-cgid \
        --enable-authn-anon --enable-authn-alias
if [ $? -ne 0 ]; then
  echo "mpmbuild prefork failed"
  exit 1
fi

# Install it
RPM_BUILD_ROOT=$output_loc
make DESTDIR=$RPM_BUILD_ROOT install
if [ $? -ne 0 ]; then
  echo "mpmbuild prefork install failed"
  exit 1
fi

# Clean it before building worker.
make clean

# For the other MPMs, just build httpd and no optional modules
# --enable-modules=none doesn't work... No problem the build takes just more time.
mpmbuild worker \
        --enable-mods-shared=all \
        ${add_conf} \
        ${add_conf_sys} \
        --enable-proxy \
        --enable-cache --enable-mem-cache \
        --enable-file-cache --enable-disk-cache \
        --enable-cgid \
        --enable-authn-anon --enable-authn-alias
if [ $? -ne 0 ]; then
  echo "mpmbuild worker failed"
  exit 1
fi

# hp-ux-9000_800 can't relocate libraries so we really install them twice...
if [ $BUILD_TAG = "hpux-parisc2" ]; then
  echo "hpux-parisc2... Pfff installing in /opt/jboss/httpd"
  rm -rf /opt/jboss/httpd
  make install
fi

# Install it (and more the prefork one).
mkdir -p $RPM_BUILD_ROOT/mod_cluster_worker
make DESTDIR=$RPM_BUILD_ROOT/mod_cluster_worker install
if [ $? -ne 0 ]; then
  echo "mpmbuild worker install failed"
  exit 1
fi
mv ${RPM_BUILD_ROOT}/${_sbindir}/httpd ${RPM_BUILD_ROOT}/${_sbindir}/httpd.prefork
cp ${RPM_BUILD_ROOT}/mod_cluster_worker/${_sbindir}/httpd ${RPM_BUILD_ROOT}/${_sbindir}/httpd
rm -rf ${RPM_BUILD_ROOT}/mod_cluster_worker

# Add the needed openssl stuff.
current_loc=`pwd`
install_loc=${output_loc}/${PREFIX}
if $has_openssl_loc; then
  echo "Adding OpenSSL libraries ..."
  (cd ${common_loc}/lib
   tar -cf ${current_loc}/x.tar engines *.${so_extension}*
  )
  (cd ${install_loc}/lib
   tar -xf ${current_loc}/x.tar
  )
  rm -f ${current_loc}/x.tar

  cp ${common_loc}/bin/openssl ${install_loc}/bin
fi

# Arrange the sl file into so on hpux
if [ "$BUILD_SYS" = "hpux" ]; then
  for file in `find ${RPM_BUILD_ROOT}/opt/jboss/httpd/lib/httpd -name *.sl`
  do
    nfile=`echo $file | sed 's:\.sl:\.so:'`
    mv $file $nfile
  done
fi

# Add default mod_cluster conf in httpd.conf
echo "Add default mod_cluster conf in httpd.conf for $build_version"
case $build_version in
  1.0.*)
    echo "skipped!"
    ;;
  1.1.*|1.2.*|1.3.*)
    if [ "$BUILD_SYS" = "hpux" ]; then
    echo "Add the mod_cluster modules now because we can't use buildconf on hpux"
    cat >> "${RPM_BUILD_ROOT}/opt/jboss/httpd/httpd/conf/httpd.conf" <<EOF

LoadModule proxy_module /opt/jboss/httpd/lib/httpd/modules/mod_proxy.so
LoadModule proxy_ajp_module /opt/jboss/httpd/lib/httpd/modules/mod_proxy_ajp.so
LoadModule proxy_http_module /opt/jboss/httpd/lib/httpd/modules/mod_proxy_http.so

LoadModule proxy_cluster_module /opt/jboss/httpd/lib/httpd/modules/mod_proxy_cluster.so

LoadModule manager_module /opt/jboss/httpd/lib/httpd/modules/mod_manager.so
LoadModule ${slotmem}_module /opt/jboss/httpd/lib/httpd/modules/mod_${slotmem}.so
LoadModule advertise_module /opt/jboss/httpd/lib/httpd/modules/mod_advertise.so

EOF
    fi
    case ${httpd_version} in
      2.2*)
        cat >> "${RPM_BUILD_ROOT}/opt/jboss/httpd/httpd/conf/httpd.conf" <<EOF
# MOD_CLUSTER_ADDS
# Adjust to you hostname and subnet.
<IfModule manager_module>
  Listen 127.0.0.1:6666
  ManagerBalancerName mycluster
  <VirtualHost 127.0.0.1:6666>
    <Location />
     Order deny,allow
     Deny from all
     Allow from 127.0.0
    </Location>

    KeepAliveTimeout 300
    MaxKeepAliveRequests 0
    #ServerAdvertise on http://@IP@:6666
    AdvertiseFrequency 5
    #AdvertiseSecurityKey secret
    #AdvertiseGroup @ADVIP@:23364
    EnableMCPMReceive

    <Location /mod_cluster_manager>
       SetHandler mod_cluster-manager
       Order deny,allow
       Deny from all
       Allow from 127.0.0
    </Location>

  </VirtualHost>
</IfModule>
EOF
        ;;
      2.4*)
        cat >> "${RPM_BUILD_ROOT}/opt/jboss/httpd/httpd/conf/httpd.conf" <<EOF
# MOD_CLUSTER_ADDS
# Adjust to you hostname and subnet.
<IfModule manager_module>
  Listen 127.0.0.1:6666
  ManagerBalancerName mycluster
  <VirtualHost 127.0.0.1:6666>
    <Location />
     Require ip 127.0.0
    </Location>

    KeepAliveTimeout 300
    MaxKeepAliveRequests 0
    #ServerAdvertise on http://@IP@:6666
    AdvertiseFrequency 5
    #AdvertiseSecurityKey secret
    #AdvertiseGroup @ADVIP@:23364
    EnableMCPMReceive

    <Location /mod_cluster_manager>
       SetHandler mod_cluster-manager
       Require ip 127.0.0
    </Location>

  </VirtualHost>
</IfModule>
EOF
        ;;
    esac
    echo "Done"
    ;;
esac

# Copy the install_home scripts
BASEHTTPD=/opt/jboss/httpd/
HTTPDSBIN=sbin
cp -p ../mod_cluster/native/scripts/*.sh ${RPM_BUILD_ROOT}/${BASEHTTPD}/${HTTPDSBIN}/

# Also build the java part.
(cd ../mod_cluster
  # Test the different profiles
  mvn -P TC6 install -Djava.awt.headless=true -Djava.net.preferIPv4Stack=true
  if [ $? -ne 0 ]; then
    echo "maven TC6 build failed"
    exit 1
  fi
  mvn clean
  mvn -P TC7 install -Djava.awt.headless=true -Djava.net.preferIPv4Stack=true
  if [ $? -ne 0 ]; then
    echo "maven TC7 build failed"
    exit 1
  fi
  mvn clean
  mvn -P AS7 install -Djava.awt.headless=true -Djava.net.preferIPv4Stack=true
  if [ $? -ne 0 ]; then
    echo "maven AS7 build failed"
    exit 1
  fi
  
  mvn clean
  mvn -P dist install -Djava.awt.headless=true -Djboss.server.log.dir=`pwd` -Djava.net.preferIPv4Stack=true
  if [ $? -ne 0 ]; then
    echo "maven dist install -Djava.awt.headless=true failed"
    exit 1
  fi
  # the src/assembly/bin.xml is executed too early so the first time the file is "empty".
  mvn -P dist package -Djava.awt.headless=true -Djboss.server.log.dir=`pwd` -Djava.net.preferIPv4Stack=true -Dmaven.test.skip=true
  if [ $? -ne 0 ]; then
    echo "maven dist package -Djava.awt.headless=true failed"
    exit 1
  fi
  cp target/*-bin.* ${build_top}/output
) || exit 1
mkdir -p ${RPM_BUILD_ROOT}/${BASEHTTPD}/tools
(cd ../mod_cluster/test/java
  mvn compile
  ant download
  (cd Clients/testhttpd
   ant build
  )
  javac *Advertize.java
  cp -p *Advertize.class ${RPM_BUILD_ROOT}/${BASEHTTPD}/tools
  cp -p output/classes/testhttpd.war ${RPM_BUILD_ROOT}/${BASEHTTPD}/tools
) || exit 1

# Install httpd using the ant script in mod_cluster
(cd ../mod_cluster/test/java
  if [ "${BUILD_SYS}" = "windows" ]; then
    BASELOC=`ant base | grep echo | sed 's:\[echo\]::' | sed 's:^ *::'`
  else
    BASELOC=`ant base | grep echo | sed 's:\[echo\]::' | sed 's:^ *::' | sed 's: :/:g'`
  fi
  echo "Running: ant installhttpd -DBUILDTEST=Y -Dbuild_version=${build_version} -Droot=${output_loc} -Dbase.apache=${BASELOC}"
  ant installhttpd -DBUILDTEST=Y -Dbuild_version=${build_version} -Droot=${output_loc} -Dbase.apache="${BASELOC}"
)

case ${httpd_version} in
  2.2*)
   apxs=sbin/apxs
   ;;
  2.4*)
   apxs=bin/apxs
   ;;
esac

# Build the *.so for any httpd-2.2.x using the httpd install done before
if [ "${BUILD_SYS}" != "windows" ]; then
  echo "Building mod_cluster modules with ${apxs}"
  BASELOC=`(cd ../mod_cluster/test/java; ant base | grep echo | sed 's:\[echo\]::' | sed 's:^ *::' | sed 's: :/:g')`
  for dir in `echo mod_manager advertise mod_${slotmem} mod_proxy_cluster`
  do
    (cd ../mod_cluster/native/$dir
     sh buildconf
     ./configure --with-apxs=${BASELOC}/opt/jboss/httpd/${apxs}
     make clean
     make
     for file in `ls *.sl *.so`
     do
       nfile=`echo $file | sed 's:\.sl:\.so:'`
       cp -p $file ../../target/$nfile
       if [ -f ${RPM_BUILD_ROOT}/${BASEHTTPD}/lib/httpd/modules/$nfile ]; then
         echo "${RPM_BUILD_ROOT}/${BASEHTTPD}/lib/httpd/modules/$nfile already there"
       else
         echo "Copying $file to ${RPM_BUILD_ROOT}/${BASEHTTPD}/lib/httpd/modules/$nfile"
         cp -p $file ${RPM_BUILD_ROOT}/${BASEHTTPD}/lib/httpd/modules/$nfile
       fi
     done
    ) || exit 1
  done
  (cd ../mod_cluster/target
   tar -cf - *.so | gzip -c > ${build_top}/output/${PACKAGE}-${build_version}-${BUILD_SYS}-${BUILD_CPU}-so.tar.gz
  ) || exit 1
fi

echo "Done"
exit 0
